"""
Compare Solvers
===============

Fixed-interval output vs. explicit output

Part 1: Moments (means, standard deviations)
Part 2: Distributions

Written by TR Maarleveld, Amsterdam, The Netherlands
E-mail: tmd200@users.sourceforge.net
Last Change: September 01, 2015
"""

import stochpy, numpy as np,os
from matplotlib import rc,gridspec

cDir = os.getcwd()                      # current working directory
model_dir = os.path.join(cDir,'models')
data_dir = os.path.join(cDir,'data')

### Figure formatting (optional) ###
rc('font',**{'family':'serif','serif':['Minion Pro']}) # set a font-family

golden_mean = (5**0.5-1.0)/2.0          # Aesthetic ratio
fig_width_mm = 130
inches_per_mm = 1.0/25.40
fig_width = fig_width_mm*inches_per_mm  # width in mm
fig_height = fig_width*golden_mean*1.25 # height in inches
fig_size = [fig_width,fig_height]

params = {
          'axes.labelsize' : 10,
          'font.size' : 10,
          'text.fontsize' : 10,
          'legend.fontsize': 9,
          'xtick.labelsize' : 9,
          'ytick.labelsize' : 9,
          'text.usetex': True,
          'figure.figsize': fig_size}        

stochpy.plt.rcParams.update(params)

################################

def ParseMoments(filename,file_dir,IsHeader = True):
    """ Parse moments from earlier done simulations """
    file_in = open(os.path.join(file_dir,filename),"r")     
    D_data = {}
    for line in file_in:         
        sline = line.strip().split('\t')
        if not IsHeader:
            L_values = map(float,sline)
            for i in xrange(len(L_values)):                
                D_data[L_column_names[i]].append(L_values[i])
        else:
            L_column_names = sline
            for name in L_column_names:
                D_data[name] = []
            IsHeader = False            
    return D_data


# Do simulations separate for plotting purposes
smod1 = stochpy.SSA() # explicit simulation
smod2 = stochpy.SSA() # frame based simulation

smod1.Model('Burstmodel.psc',model_dir)
smod2.Model('Burstmodel.psc',model_dir)

### Part 1: Moments ###

D_data = ParseMoments('simulation_moments.dat',data_dir)

gs = gridspec.GridSpec(3,2)
ax1 = stochpy.plt.subplot(gs[0, :])
ax1.errorbar(x=D_data['frames'],y=D_data['mu(frame)/mu'], yerr=D_data['SD(mu_c)/Mean(mu_s)'],fmt='o',color= '#0072B2')
ax1.errorbar(x=D_data['frames'],y=D_data['sigma(frame)/sigma'], yerr=D_data['SD(sigma_c)/Mean(sigma_s)'],fmt='v', color= '#009E73')
ax1.legend([r'$\mu$',r'$\sigma$'],numpoints=1,frameon=False)
stochpy.plt.xscale('log')
stochpy.plt.xlim([1,D_data['frames'][-1]*10])
stochpy.plt.ylabel(r'Ratio ($\mu$,$\sigma$)')
stochpy.plt.ylim([0,2])
stochpy.plt.setp(ax1.get_xticklabels(), visible=False)

ax2 = stochpy.plt.subplot(gs[1, :])
ax2.errorbar(x=D_data['frames'],y=D_data['T(frame)/T(real)'],fmt='s',color='#D55E00')
stochpy.plt.xscale('Log')
stochpy.plt.xlabel('Number of fixed intervals')
stochpy.plt.xlim([1,D_data['frames'][-1]*10])
stochpy.plt.ylabel(r'Ratio ($t$)')
stochpy.plt.ylim([0,0.4])
stochpy.plt.yticks([0,0.1,0.2,0.3,0.4])

### Part 2: Distributions ###

### Settings ###
ntrajectories = 10 # more trajectories can give a more accurate figure (10 was used in the manuscript)
L_nframes = [10**4,10**6]
nsteps = 10**6
###############

smod1.DoStochSim(end=nsteps,mode='steps',trajectories=ntrajectories)
smod1.GetAverageSpeciesDistributions()

ax3 = stochpy.plt.subplot(gs[2,0])
smod1.PlotAverageSpeciesDistributionsConfidenceIntervals(species2plot='mRNA',nstd=1.96,IsLegend=False,colors='#0072B2')
smod2.DoStochKitStochSim(endtime=smod1.data_stochsim.simulation_endtime,frames=L_nframes[0],trajectories=ntrajectories)
smod2.GetAverageSpeciesDistributions()
smod2.PlotAverageSpeciesDistributions(species2plot='mRNA',colors='#D55E00',marker='.',nstd=1.96,title='',IsLegend=False)
stochpy.plt.xlabel('mRNA/cell')
stochpy.plt.xlim([0,50])
stochpy.plt.ylim([0,0.015])
stochpy.plt.yticks([0,0.005,0.010,0.015])

ax4 = stochpy.plt.subplot(gs[2,1])
smod1.plot.ResetPlotnum()
smod2.plot.ResetPlotnum()
smod1.PlotAverageSpeciesDistributionsConfidenceIntervals(species2plot='mRNA',nstd=1.96,IsLegend=False,colors='#0072B2')
smod2.DoStochKitStochSim(endtime=smod1.data_stochsim.simulation_endtime,frames=L_nframes[1],trajectories=ntrajectories)
smod2.GetAverageSpeciesDistributions()
smod2.PlotAverageSpeciesDistributions(species2plot='mRNA',colors='#D55E00',marker='.',nstd=1.96,title='',IsLegend=False)
stochpy.plt.xlabel('mRNA/cell')
stochpy.plt.xlim([0,50])
stochpy.plt.ylim([0,0.015])
stochpy.plt.ylabel('')
stochpy.plt.setp(ax4.get_yticklabels(), visible=False)
stochpy.plt.savefig('Figure_2.pdf')
