"""
Modeling the immigration death model: ---> mRNA --->

Here, we compare the outcome of stochastic simulations vs. the analytical solution. Must code is used to make nice figures rather than doing actual stochastic simulations.

Written by TR Maarleveld, Amsterdam, The Netherlands
E-mail: tmd200@users.sourceforge.net
Last Change: September 01, 2015
"""

import os,mpmath,stochpy,numpy as np,os
from matplotlib import rc,gridspec

cDir = os.getcwd()                      # current working directory
model_dir = os.path.join(cDir,'models')

### Figure formatting (optional) ###
rc('font',**{'family':'serif','serif':['Minion Pro']}) # set a font-family
rc('text', usetex=True)                 # allow tex commands

golden_mean = (5**0.5-1.0)/2.0          # Aesthetic ratio
fig_width_mm = 130
inches_per_mm = 1.0/25.40
fig_width = fig_width_mm*inches_per_mm  # width in mm
fig_height = fig_width*golden_mean*1.9  # height in inches
fig_size = [fig_width,fig_height]

params = {
          'axes.labelsize' : 10,
          'font.size' : 10,
          'text.fontsize' : 10,
          'legend.fontsize': 9,
          'xtick.labelsize' : 9,
          'ytick.labelsize' : 9,
          'text.usetex': True,
          'figure.figsize': fig_size}      

stochpy.plt.rcParams.update(params)

####################################

def GetAnalyticalAutocorrelation(nlags,kdeg):
    """    
    Gardiner - Handbook of Stochastic Methods 1997 page 64    
    --> X -->
    
    Input:
    - *nlags* (integer)
    - *kdeg* (float)  
    """
    L_taus = np.linspace(0,nlags,nlags*100) # times 100 to create enough points for a solid line
    L_acor = np.exp(-kdeg*L_taus)
    return (L_taus,L_acor)    
   
def GetPoissonDistribution(Lambda,nmin,nmax):
    L_dist = []
    for n in np.linspace(nmin,nmax):
        L_dist.append(((mpmath.power(Lambda,n))*np.exp(-Lambda))/mpmath.factorial(n))
    return L_dist
    
def GetDistortedPoissonDistribution(Lambda,ymin,ymax,kdeg):
    L_dist = []   
    for y in np.linspace(ymin,ymax):
        L_dist.append(((mpmath.power(Lambda,y/kdeg))*np.exp(-Lambda))/mpmath.factorial(y/kdeg))
    return L_dist        

smod = stochpy.SSA(model_file='ImmigrationDeath.psc',dir=model_dir,IsQuiet=True) # Start SSA module and parse the immigration death model
gs = gridspec.GridSpec(3, 2,height_ratios=[0.5,1,1])  # Define the grid specifications
# Figure A
ax1 = stochpy.plt.subplot(gs[0, 0])   
smod.DoStochSim(end=10**3,IsTrackPropensities=True) 
smod.PlotSpeciesTimeSeries(title='',colors= ['#0072B2'],IsLegend=False)
stochpy.plt.xlabel('Time (min)')
stochpy.plt.xlim([0,50])
stochpy.plt.ylabel(r'$n_{mRNA}$')

################# Set y-ticks #################
dif1 = max(smod.data_stochsim.species[:,0]) - 50 # mean = 50
dif2 = 50 - min(smod.data_stochsim.species[:,0])
if dif1 > dif2:
    dif = np.ceil(dif1)
else:
    dif = np.ceil(dif2)
stochpy.plt.yticks([50-dif,50,50+dif])
###############################################

# Figure B
smod.plot.plotnum = 1 # reset 'figure' number
ax2 = stochpy.plt.subplot(gs[0, 1])    
smod.PlotPropensitiesTimeSeries(title='',colors= ['#0072B2','#009E73'],IsLegend=False)
stochpy.plt.xlabel('Time (min)')
stochpy.plt.xlim([0,50])

################# Set y-ticks #################
dif1 = max(smod.data_stochsim.propensities[:,1]) - 10 # mean = 10
dif2 = 10 - min(smod.data_stochsim.propensities[:,1])
if dif1 > dif2:
    dif = np.ceil(dif1)
else:
    dif = np.ceil(dif2)
stochpy.plt.yticks([10-dif,10,10+dif])
###############################################

# Figure C
smod.plot.plotnum = 1
ax3 = stochpy.plt.subplot(gs[1, 0])   
smod.DoStochSim(end=10**6,IsTrackPropensities=True)
smod.PlotSpeciesDistributions(linestyle= 'solid',linewidth = 2,colors= ['#0072B2'],title='',IsLegend=False)
L_dist = GetPoissonDistribution(Lambda = 50,nmin = min(smod.data_stochsim.species[:,0]),nmax= max(smod.data_stochsim.species[:,0]))
stochpy.plt.plot(np.linspace(min(smod.data_stochsim.species[:,0]),max(smod.data_stochsim.species[:,0])),L_dist,color ='k')
stochpy.plt.xlabel(r'$n_{mRNA}$')
stochpy.plt.ylabel('PMF')
stochpy.plt.yticks([0,0.02,0.04,0.06])

# Figure D
smod.plot.plotnum = 1
ax4 = stochpy.plt.subplot(gs[1, 1])
smod.PlotPropensitiesDistributions(rates2plot= 'R2',linestyle= 'solid',linewidth = 2, colors= ['#009E73'],title='',IsLegend=False,bin_size=0.2)
L_dist = GetDistortedPoissonDistribution(50,min(smod.data_stochsim.propensities[:,1]),max(smod.data_stochsim.propensities[:,1]),0.2)
stochpy.plt.plot(np.linspace(min(smod.data_stochsim.propensities[:,1]),max(smod.data_stochsim.propensities[:,1])),np.array(L_dist),color ='k')

stochpy.plt.xlabel('mRNA degradation propensity')
stochpy.plt.ylabel('PMF')
stochpy.plt.yticks([0,0.02,0.04,0.06])

# Figure E   
smod.plot.plotnum = 1      
ax5 = stochpy.plt.subplot(gs[2, :])     
endtime = 100000 # simulation time scales with the endtime (100000 was used in the manuscript)
N = 50
ksyn = 10
L_kdegs = [0.01,0.025,0.05,0.1,0.5]
L_markers = ['o','s','v','*','p']
L_colors = ['#0072B2','#D55E00','#009E73','#CC79A7','#F0E442'] # b,r,g,m,y
smod.ChangeParameter("Ksyn",ksyn) # modify parameter
i=0
for kdeg in L_kdegs:
    smod.ChangeParameter("Kdeg",kdeg)
    smod.DoStochSim(end=endtime,mode='time')
    smod.GetSpeciesAutocorrelations(n_samples = endtime)
    smod.PlotSpeciesAutocorrelations(nlags=N,marker = L_markers[i],colors=L_colors[i],IsLegend=False, title='')
    smod.plot.plotnum = 1
    L_taus,L_acor = GetAnalyticalAutocorrelation(nlags=N,kdeg = kdeg)
    stochpy.plt.step(L_taus,L_acor,color ='k')
    stochpy.plt.xlabel('Lag (min)')
    i+=1
stochpy.plt.tight_layout()
stochpy.plt.savefig('Figure_3_thesis.pdf')
