"""
Burstmodel time series analysis

Here, we compare the outcome of stochastic simulations vs. the analytical solution. Must code is used to make nice figures rather than doing actual stochastic simulations.

Written by TR Maarleveld, Amsterdam, The Netherlands
E-mail: tmd200@users.sourceforge.net
Last Change: September 01, 2015
"""

import numpy as np,mpmath,sys,os, stochpy
from matplotlib import rc,gridspec

cDir = os.getcwd()                      # current working directory
model_dir = os.path.join(cDir,'models')

### Figure formatting (optional) ###
rc('font',**{'family':'serif','serif':['Minion Pro']}) # set a font-family
rc('text', usetex=True)                 # allow tex commands

golden_mean = (5**0.5-1.0)/2.0          # Aesthetic ratio
fig_width_mm = 130
inches_per_mm = 1.0/25.40
fig_width = fig_width_mm*inches_per_mm  # width in mm
fig_height = fig_width*golden_mean*1  # height in inches
fig_size = [fig_width,fig_height]

params = {
          'axes.labelsize' : 10,
          'font.size' : 10,
          'text.fontsize' : 10,
          'legend.fontsize': 9,
          'xtick.labelsize' : 9,
          'ytick.labelsize' : 9,
          'text.usetex': True,
          'figure.figsize': fig_size}      
  

stochpy.plt.rcParams.update(params)
stochpy.plt.axes([0.125,0.2,0.95-0.125,0.95-0.2])

####################################

### Simulation
endtime = 100
smod = stochpy.SSA()
smod.Model('Burstmodel.psc',dir=model_dir)
smod.ChangeParameter('koff',0.05)
smod.ChangeParameter('kon',0.05)
smod.DoStochSim(end=endtime,mode='time')

### Figure ###
gs = gridspec.GridSpec(4, 1,width_ratios=[1],height_ratios=[0.3,1,0.3,1])
ax1 = stochpy.plt.subplot(gs[0])
smod.PlotSpeciesTimeSeries(species2plot = 'ONstate',colors = ['#0072B2'],IsLegend=False, title='')
stochpy.plt.xlim([0,endtime])
stochpy.plt.xticks([])
stochpy.plt.ylim([0,1.1])
stochpy.plt.yticks([])
stochpy.plt.xlabel('')
stochpy.plt.ylabel('')
stochpy.plt.text(-8.5,0.9,'ON')
stochpy.plt.text(-8.5,0,'OFF')

ax2 = stochpy.plt.subplot(gs[1])
smod.plot.plotnum = 1
smod.PlotSpeciesTimeSeries(species2plot = 'mRNA',colors = ['#009E73'],IsLegend=False, title='')
stochpy.plt.xlim([0,endtime])
stochpy.plt.xticks([])
stochpy.plt.title('')
stochpy.plt.xlabel('')
stochpy.plt.ylabel(r'$n_{mRNA}$')
yticks = [0,20,40,60]
stochpy.plt.yticks(yticks)

smod.ChangeParameter('koff',5.0)
smod.ChangeParameter('kon',5.0)
smod.DoStochSim(end=endtime,mode='time')

ax3 = stochpy.plt.subplot(gs[2])
smod.plot.plotnum = 1
smod.PlotSpeciesTimeSeries(species2plot = 'ONstate',colors = ['#0072B2'],IsLegend=False, title='')
stochpy.plt.xlim([0,endtime])
stochpy.plt.xticks([])
stochpy.plt.ylim([0,1.1])
stochpy.plt.yticks([])
stochpy.plt.xlabel('')
stochpy.plt.ylabel('')
stochpy.plt.text(-8.5,0.9,'ON')
stochpy.plt.text(-8.5,0,'OFF')

ax4 = stochpy.plt.subplot(gs[3])
smod.plot.plotnum = 1
smod.PlotSpeciesTimeSeries(species2plot = 'mRNA',colors = ['#D55E00'],IsLegend=False, title='')
stochpy.plt.xlim([0,endtime])
stochpy.plt.title('')
stochpy.plt.xlabel('Time (seconds)')
stochpy.plt.ylabel(r'$n_{mRNA}$')
yticks = [0,20,40,60]
stochpy.plt.yticks(yticks)

stochpy.plt.savefig('cs2_Figure_4.pdf')
