"""
Modeling bursty and non-bursty transcription

Written by TR Maarleveld, Amsterdam, The Netherlands
E-mail: tmd200@users.sourceforge.net
Last Change: September 01, 2015
"""

import numpy as np,mpmath,sys,stochpy,os
from matplotlib import rc,gridspec
mpmath.mp.pretty = True

cDir = os.getcwd()                      # current working directory
data_dir = os.path.join(cDir,'data')

rc('font',**{'family':'serif','serif':['Minion Pro']}) # set a font-family
rc('text', usetex=True)                 # allow tex commands

golden_mean = (5**0.5-1.0)/2.0          # Aesthetic ratio
fig_width_mm = 130
inches_per_mm = 1.0/25.40
fig_width = fig_width_mm*inches_per_mm  # width in mm
fig_height = fig_width*golden_mean*1  # height in inches
fig_size = [fig_width,fig_height]

params = {
          'axes.labelsize' : 10,
          'font.size' : 10,
          'text.fontsize' : 10,
          'legend.fontsize': 9,
          'xtick.labelsize' : 9,
          'ytick.labelsize' : 9,
          'text.usetex': True,
          'figure.figsize': fig_size}      
  
stochpy.plt.rcParams.update(params)
stochpy.plt.axes([0.125,0.2,0.95,0.95])

####################################

def GetAnalyticalPDF(kon,koff,keg,ksyn):
    """ Get the analytical probability density function. The analytical solution is taken from Sharezaei and Swain 2008 - Analytical distributions for stochastic gene expression """
    x_values = np.linspace(0,50,10000)
    y_values = []
    for m in x_values:
        a = ((ksyn/kdeg)**m)*np.exp(-ksyn/kdeg)/mpmath.factorial(m)
        b = mpmath.mp.gamma((kon/kdeg)+m) * mpmath.mp.gamma(kon/kdeg + koff/kdeg)/ (mpmath.mp.gamma(kon/kdeg + koff/kdeg + m)* mpmath.mp.gamma(kon/kdeg))
        c = mpmath.mp.hyp1f1(koff/kdeg,kon/kdeg + koff/kdeg + m,ksyn/kdeg)
        y_values.append(a*b*c)
    return x_values,y_values

def GetAnalyticalWaitingtimes(kon,koff,ksyn):
    """ Get analytical waiting times """
    A = mpmath.sqrt(-4*ksyn*kon+(koff + kon + ksyn)**2)
    x = []
    for i in np.linspace(-20,5,5000):
        x.append(mpmath.exp(i))     
    y = []
    for t in x:
        B = koff + ksyn - (mpmath.exp(t*A)*(koff+ksyn-kon))-kon+A+ mpmath.exp(t*A)*A
        p01diff = mpmath.exp(-0.5*t*(koff + kon + ksyn+A))*B/(2.0*A)
        y.append(p01diff*ksyn)        
    return (x,y)

def ParseDistributionData(filename):
    try:
        file_in = open(os.path.join(data_dir,filename),'r')
    except:
        raise Warning
    n=0
    dist = []
    for line in file_in:
        if line.startswith("Position"):
            n+=1        
        elif n==3:
            sline = line.strip('\n')        
            data = sline.split("\t")
            data[0] = int(data[0])
            data[1] = float(data[1])        
            dist.append(data)
    dist = np.array(dist)
    return dist
    
def ParseWaitingtimesData(filename):
    try:
        file_in = open(os.path.join(data_dir,filename),'r')
    except:
        raise Warning
    IsR3 = False
    waitingtimes = []
    for line in file_in:    
        if "R3" in line:
            IsR3 = True
            continue
        elif "R4" in line:
            IsR3 = False
        if IsR3:
            sline = line.strip("\n")
            waitingtimes.append(float(sline)) 
    return waitingtimes

L_dist_005 = ParseDistributionData("Distributions0.05.txt")
L_dist_5 = ParseDistributionData("Distributions5.txt")

L_wait_005 = ParseWaitingtimesData("Waitingtimes0.05.txt")
L_wait_5  = ParseWaitingtimesData("Waitingtimes5.txt")

gs = gridspec.GridSpec(2, 1, width_ratios=[1], height_ratios=[1,1])

ax1 = stochpy.plt.subplot(gs[0])
stochpy.plt.step(L_dist_005[:,0]+0.5,L_dist_005[:,1],color= '#009E73',lw=2)
stochpy.plt.step(L_dist_5[:,0]+0.5,L_dist_5[:,1],color = '#D55E00',lw = 2)  

kon = 0.05
koff = 0.05
kdeg = 2.5
ksyn = 80.0
x,y = GetAnalyticalPDF(kon,koff,kdeg,ksyn)
stochpy.plt.step(x,y,color ='k')

kon = 5.0
koff = 5.0
x,y = GetAnalyticalPDF(kon,koff,kdeg,ksyn)
stochpy.plt.step(x,y,color ='k')

stochpy.plt.xlabel(r'Molecules/cell$^{-1}$')
stochpy.plt.ylabel('PMF')
stochpy.plt.text(37, .025, r'Bimodal') # bursts
stochpy.plt.text(7,0.025, r'Unimodal')
stochpy.plt.xlim([0,50])
stochpy.plt.xticks([0,10,20,30,40,50])
stochpy.plt.ylim([0,0.05])
stochpy.plt.yticks([0,0.01,0.02,0.03,0.04,0.05])

(x1,y1,nbins) = stochpy.modules.Analysis.LogBin(L_wait_005,1.5)
(x2,y2,nbins) = stochpy.modules.Analysis.LogBin(L_wait_5,1.5)

kon = 0.05
koff = 0.05
(x3,y4) = GetAnalyticalWaitingtimes(kon,koff,ksyn)

kon = 5.0
koff = 5.0
(x5,y6) = GetAnalyticalWaitingtimes(kon,koff,ksyn)

ax2 = stochpy.plt.subplot(gs[1])
stochpy.plt.loglog(x1,y1,linestyle = 'None',marker='o',color= '#009E73')
stochpy.plt.loglog(x2,y2,linestyle = 'None',marker='v',color= '#D55E00')
stochpy.plt.loglog(x3,y4,color ='k')
stochpy.plt.loglog(x5,y6,color ='k')

stochpy.plt.xlabel('Molecules/cell synthesis waiting times (min)')
stochpy.plt.ylabel('PDF')
stochpy.plt.xlim([10**-7,10**3])
stochpy.plt.ylim([10**-7,10**3])
stochpy.plt.text(0.0005, .005, r'Bimodal') 
stochpy.plt.text(2, .005, r'Unimodal') # bursts
stochpy.plt.xticks([10**-7,10**-5,10**-3,10**-1,10**1,10**3])
stochpy.plt.yticks([10**-7,10**-5,10**-3,10**-1,10**1,10**3])

stochpy.plt.savefig('Figure_5.pdf')
