"""
Modeling single-molecule enzymology

Example model: E+S <--> ES --> E + P

Written by TR Maarleveld, Amsterdam, The Netherlands
E-mail: tmd200@users.sourceforge.net
Last Change: September 01, 2015
"""

import stochpy,numpy as np,os
from matplotlib import rc,gridspec

cDir = os.getcwd()                      # current working directory
model_dir = os.path.join(cDir,'models')

### Figure formatting (optional) ###
rc('font',**{'family':'serif','serif':['Minion Pro']}) # set a font-family
rc('text', usetex=True)                 # allow tex commands

golden_mean = (5**0.5-1.0)/2.0          # Aesthetic ratio
fig_width_mm = 130
inches_per_mm = 1.0/25.40
fig_width = fig_width_mm*inches_per_mm  # width in mm
fig_height = fig_width*golden_mean*1.15 # height in inches
fig_size = [fig_width,fig_height]

params = {
          'axes.labelsize' : 10,
          'font.size' : 10,
          'text.fontsize' : 10,
          'legend.fontsize': 9,
          'xtick.labelsize' : 9,
          'ytick.labelsize' : 9,
          'text.usetex': True,
          'figure.figsize': fig_size}    
stochpy.plt.rcParams.update(params)

####################################

def getFt(t):
    return (np.exp(-0.5*(k1f*S + k1b+k2f + np.sqrt(-4*k1f*k2f*S + (k1f*S + k1b+k2f)**2))*t)*(-1+np.exp(np.sqrt(-4*k1f*k2f*S+(k1f*S + k1b+k2f)**2)*t))*k1f*k2f*S)/np.sqrt(-4*k1f*k2f*S+(k1f*S + k1b+k2f)**2)

smod = stochpy.SSA()
smod.Model('EnzymeKinetics.psc',dir = model_dir)

### Model settings ###

S = 10
k1f = 1/15.0
k2f = 0.75
k1b = 1/12.0

smod.ChangeParameter('k1f',k1f)
smod.ChangeParameter('k1b',k1b)
smod.ChangeParameter('k2f',k2f)
smod.ChangeInitialSpeciesAmount('S',S)

sim_endtime = 125

### Characteristics ###

Vss = k2f*S/(S+(k1b+k2f)/k1f)
Wt = (k1b+k2f)/(k1f*k2f*S) + 1.0/k2f
Vmax = k2f
Km = (k1b+k2f)/k1f

### Plotting ###

gs = gridspec.GridSpec(3, 2,height_ratios=[0.3,0.3,1]) 
smod.DoStochSim(end=sim_endtime,mode='time',trajectories=1)
smod.plot.plotnum = 1  
ax1 = stochpy.plt.subplot(gs[0, :])
smod.PlotSpeciesTimeSeries(species2plot='E',colors=['#009E73'],title='',IsLegend=False)
stochpy.plt.xticks([])
stochpy.plt.xlabel('')
stochpy.plt.xlim([0,sim_endtime])
stochpy.plt.ylabel(r'$E$')
stochpy.plt.yticks([0,1])
stochpy.plt.ylim([0,1.1])

smod.plot.plotnum = 1   
ax2 = stochpy.plt.subplot(gs[1, :])
smod.PlotSpeciesTimeSeries(species2plot='ES',colors=['#D55E00'],title='',IsLegend=False)
stochpy.plt.xlim([0,sim_endtime])
stochpy.plt.xlabel('Time (min)')
stochpy.plt.ylabel(r'$ES$')
stochpy.plt.yticks([0,1])
stochpy.plt.ylim([0,1.1])

smod.plot.plotnum = 1  
ax3 = stochpy.plt.subplot(gs[2, 0])

smod.DoStochSim(end=sim_endtime,mode='time',trajectories=3)
smod.PlotSpeciesTimeSeries(species2plot='P',colors=['#0072B2'],title='',IsLegend=False)
t = np.linspace(0,smod.data_stochsim.simulation_endtime)
stochpy.plt.plot(t,Vss*t,color ='k')
stochpy.plt.xlabel('Time (min)')
stochpy.plt.ylabel(r'$P$')
stochpy.plt.ylim([0,50])

smod.DoStochSim(end=10**6,mode='steps',trajectories=1)
smod.GetWaitingtimes()

Wt_max = max(smod.data_stochsim.waiting_times['R3'])
Wt_min = min(smod.data_stochsim.waiting_times['R3'])

np.log10(Wt_min),np.log10(Wt_max)
t = np.logspace(np.log10(Wt_min),np.log10(Wt_max),100)
Ft = getFt(t)
smod.plot.plotnum = 1  
ax4 = stochpy.plt.subplot(gs[2, 1])

smod.PlotWaitingtimesDistributions('R3',colors=['#0072B2'],title='',IsLegend=False,xlabel=r'$P$ synthesis waiting times (min)')
stochpy.plt.plot(t,Ft,color ='k')
stochpy.plt.xlim([10**-2,10**2])
stochpy.plt.ylim([10**-6,10**0])
stochpy.plt.tight_layout()

stochpy.plt.savefig('cs3_Figure_6_thesis.pdf')
