 #! /usr/bin/env python
"""
Small wrapper to perform sequential simulations in an interactive manner in StochPy

Written by TR Maarleveld, Amsterdam, The Netherlands
E-mail: tmd200@users.sourceforge.net
Last Change: September 01, 2015
"""

import stochpy, numpy as np,copy,os
from matplotlib import rc,gridspec

cDir = os.getcwd()                      # current working directory
model_dir = os.path.join(cDir,'models')

### Figure formatting (optional) ###
rc('font',**{'family':'serif','serif':['Minion Pro']}) # set a font-family
rc('text', usetex=True)                 # allow tex commands

golden_mean = (5**0.5-1.0)/2.0          # Aesthetic ratio
fig_width_mm = 130
inches_per_mm = 1.0/25.40
fig_width = fig_width_mm*inches_per_mm  # width in mm
fig_height = fig_width*golden_mean*1.25 # height in inches
fig_size = [fig_width,fig_height]

params = {
          'axes.labelsize' : 10,
          'font.size' : 10,
          'text.fontsize' : 10,
          'legend.fontsize': 9,
          'xtick.labelsize' : 9,
          'ytick.labelsize' : 9,
          'text.usetex': True,
          'figure.figsize': fig_size}        

stochpy.plt.rcParams.update(params)

####################################

def doSequentialSim(smod,n_generations,interdivision_times):    
    for i in xrange(1,n_generations):     
        ### divide each species between two daughter cells ###        
        for j in xrange(0,len(smod.data_stochsim.species_labels)): 
            species_amount = smod.SSA.sim_output[-2][1:][j]                 
            if species_amount:
                smod.settings.X_matrix[j] = np.random.binomial(n=species_amount,p=0.5,size=1)                
        ### replace last time point with species amounts after division ###
        species_after_division = copy.deepcopy(list(smod.settings.X_matrix))
        species_after_division.insert(0,interdivision_times[0:i].sum()) # add time of cell division
        species_after_division.append(np.NAN) # no specific reaction occured at cell division
        smod.SSA.sim_output[-1] = copy.deepcopy(species_after_division)                         
        ### Set settings for new simulation and simulate the next generation ### 
        smod.settings.starttime = copy.deepcopy(smod.SSA.sim_output[-1][0])
        smod.settings.endtime = smod.settings.starttime + interdivision_times[i]
        smod.SSA.Execute(smod.settings) # do stochastic simulation     
    smod.FillDataStochsim() # add all data to data_stochsim object

smod1 = stochpy.SSA(model_file='celldivision_implicit.psc',dir=model_dir)
smod2 = stochpy.SSA(model_file='celldivision_explicit.psc',dir=model_dir)
generation_time = 60

### 1 Implicit model: Do short simulations for a time course plot ###
n_generations=10
smod1.DoStochSim(end=generation_time*n_generations,mode='time')

# Use a grid to plot figures
gs = gridspec.GridSpec(4,2,width_ratios=[1,1],height_ratios=[1,1,1,2])
ax1 = stochpy.plt.subplot(gs[0,-2])
smod1.PlotSpeciesTimeSeries(species2plot=['TFactive','TF'],title='',colors = ['#0072B2','#009E73'],IsLegend=False)
stochpy.plt.xlabel('')
stochpy.plt.ylabel('TF')
stochpy.plt.xticks([])
stochpy.plt.xlim([0,600])
stochpy.plt.ylim([0,25])
stochpy.plt.yticks([0,5,10,15,20])

ax2 = stochpy.plt.subplot(gs[1,-2])
smod1.plot.ResetPlotnum()
smod1.PlotSpeciesTimeSeries(species2plot=['mRNA'],title='',colors = '#0072B2',IsLegend=False)
stochpy.plt.xlabel('')
stochpy.plt.ylabel('mRNA')
stochpy.plt.xticks([])
stochpy.plt.xlim([0,600])
stochpy.plt.ylim([0,25])
stochpy.plt.yticks([0,5,10,15,20])

ax3 = stochpy.plt.subplot(gs[2,-2])
smod1.plot.ResetPlotnum()
smod1.PlotSpeciesTimeSeries(species2plot=['Protein'],title='',colors = '#0072B2',IsLegend=False)
stochpy.plt.xlabel('Time (min)')
stochpy.plt.ylabel('Protein')
stochpy.plt.xticks([0,150,300,450,600])
stochpy.plt.xlim([0,600])
stochpy.plt.ylim([0,1000])
stochpy.plt.yticks([0,200,400,600,800])

### 2 Explicit model: Do short simulations for a time course plot ###

interdivision_times = abs(np.random.gamma(generation_time,1,size=n_generations))  # cell division times with a gamma distribution
smod2.DoStochSim(end=interdivision_times[0],mode='time')
doSequentialSim(smod2,n_generations,interdivision_times)

ax4 = stochpy.plt.subplot(gs[0,-1])
smod2.PlotSpeciesTimeSeries(species2plot=['TFactive','TF'],title='',colors = ['#0072B2','#009E73'],IsLegend=False)
stochpy.plt.xlabel('')
stochpy.plt.ylabel('')
stochpy.plt.xticks([])
stochpy.plt.xlim([0,600])
stochpy.plt.ylim([0,25])
stochpy.plt.yticks([])

ax5 = stochpy.plt.subplot(gs[1,-1])
smod2.plot.ResetPlotnum()
smod2.PlotSpeciesTimeSeries(species2plot=['mRNA'],title='',colors = '#0072B2',IsLegend=False)
stochpy.plt.xlabel('')
stochpy.plt.ylabel('')
stochpy.plt.xticks([])
stochpy.plt.xlim([0,600])
stochpy.plt.ylim([0,25])
stochpy.plt.yticks([])

ax6 = stochpy.plt.subplot(gs[2,-1])
smod2.plot.ResetPlotnum()
smod2.PlotSpeciesTimeSeries(species2plot=['Protein'],title='',colors = '#0072B2',IsLegend=False)
stochpy.plt.xlabel('Time (min)')
stochpy.plt.ylabel('')
stochpy.plt.xticks([0,150,300,450,600])
stochpy.plt.xlim([0,600])
stochpy.plt.ylim([0,1000])
stochpy.plt.yticks([0,])

gs2 = gridspec.GridSpec(4,2,width_ratios=[1,1],height_ratios=[1,1,1,2]) 
gs2.update(hspace=0.25)
ax7 = stochpy.plt.subplot(gs2[3,:])
smod1.plot.ResetPlotnum()

### 3 Implicit model: Do long simulation for probability density function of protein copy numbers ###
n_generations=1000  # simulation time scales with the number of generations (2500 was used in the manuscript)
smod1.DoStochSim(end=generation_time*n_generations,mode='time')
smod1.PlotSpeciesDistributions(species2plot='Protein',linestyle='solid',title='',colors = '#0072B2',IsLegend=False)

### 4 Implicit model: Do long simulation for probability density function of protein copy numbers ###
interdivision_times = abs(np.random.gamma(generation_time,1,size=n_generations))  # cell division times with a gamma distribution
smod2.DoStochSim(end=interdivision_times[0],mode='time')
doSequentialSim(smod2,n_generations,interdivision_times)

smod2.plot.ResetPlotnum()
smod2.PlotSpeciesDistributions(species2plot='Protein',linestyle='solid',colors =  '#D55E00',title='',IsLegend=False)

stochpy.plt.xlim([0,1000])
stochpy.plt.xlabel('Protein copy number/cell')
stochpy.plt.ylabel('Propensity')
stochpy.plt.yticks([0,0.002,0.004,0.006,0.008])
stochpy.plt.text(560,0.004,'Implicit (L)',fontsize = 10)
stochpy.plt.text(660,0.002,'Explicit (R)',fontsize = 10)
stochpy.plt.savefig('Figure_7.pdf')
